<?php
/**
 * @file
 * Survey Builder interface code
 */

/**
 * Create/edit survey page title callback
 */
function survey_builder_create_title($form_id) {
  if ($form_id != SURVEY_BUILDER_NEW) {
    return t('Edit Survey');
  }
  return t('Add Survey');
}

/**
 * Create/edit survey page callback
 */
function survey_builder_create($form_id) {

  drupal_add_css(drupal_get_path('module', 'survey_builder') . '/theme/css/survey.css', array('group' => CSS_DEFAULT));

  $options = array('type' => 'file', 'scope' => 'footer');
  drupal_add_js(drupal_get_path('module', 'survey_builder') . '/theme/js/jquery.tools.min.js', $options);
  drupal_add_js(drupal_get_path('module', 'survey_builder') . '/theme/js/survey.js', $options);

  module_load_include('inc', 'form_builder', 'includes/form_builder.admin');
  module_load_include('inc', 'form_builder', 'includes/form_builder.cache');

  return form_builder_interface('survey', $form_id);

}

/**
 * Prepare a field from a form builder form array for loading
 */
function survey_builder_load_process_field(&$field, $survey_response_id = FALSE) {

  module_load_include('inc', 'survey_builder', 'includes/survey_builder.question');

  if (isset($field['#type']) && $field['#type'] == 'fieldset') {
    // Recursively process any fields within the fieldset
    foreach ($field as $key => &$value) {
      if (strpos($key, '#') !== 0) {
        survey_builder_load_process_field($value, $survey_response_id);
      }
    }
  }
  else {
    $question = question_load($field['#question_id']);
    if (!$question) {
      return;
    }

    // Merge question values into field
    $question->field['#title'] =  $question->label;
    $field = array_merge($question->field, $field);

    // If a response ID has been supplied then attempt to load this question's response
    if ($survey_response_id !== FALSE) {

      $response = question_response_load($survey_response_id, $field['#question_id']);
      if (!$response) {
        watchdog('survey_builder', 'Unable to load response for question: %id',
          array('%id' => $field['#question_id']), WATCHDOG_ERROR);
        return;
      }

      if (is_array($response)) {
        $values = array();
        foreach ($response as $value) {
          $values[] = $value->value;
        }
        $field['#default_value'] = $values;
      }
      else {
        $field['#default_value'] = $response->value;
      }

    }

  }

}

/**
 * Survey save callback
 */
function survey_builder_save($form_id) {

  module_load_include('inc', 'form_builder', 'includes/form_builder.api');
  module_load_include('inc', 'form_builder', 'includes/form_builder.cache');

  $form = form_builder_cache_load('survey', $form_id);
  form_builder_save_form($form, 'survey', $form_id);

  drupal_goto('surveys');

}

/**
 * Prepare a field from a form builder form array for saving
 */
function survey_builder_save_process_field(&$field) {

  module_load_include('inc', 'survey_builder', 'includes/survey_builder.question');

  if ($field['#type'] == 'fieldset') {
    // Recursively process any fields within the fieldset
    foreach ($field as $key => &$value) {
      if (strpos($key, '#') !== 0) {
        survey_builder_save_process_field($value);
      }
    }
  }
  else {
    $label = FALSE;

    if (!isset($field['#question_id'])) {
      $question_field = $field;
      foreach ($question_field as $key => $value) {
        if (!in_array($key, array('#type', '#options'))) {
          unset($question_field[$key]);
        }
      }

      // New question. Save it and store the new question's ID
      $id = question_save($field['#title'], $question_field);
      if ($id === FALSE) {
        watchdog('survey_builder', 'Unable to save new question: %field',
          array('%field' => print_r($field, TRUE)), WATCHDOG_ERROR);
        continue;
      }
    }
    else {
      // Pre-existing question. Check if the label should be overwritten
      $id = $field['#question_id'];
      $question = question_load_multiple(array($id), array('label' => $field['#title']));
      if (count($question) == 0) {
        $label = $field['#title'];
      }

    }

    $options = (isset($field['#options'])) ? $field['#options'] : FALSE;

    $field = array(
      '#question_id' => $id,
      '#key' => $field['#key'],
      '#form_builder' => array(
        'element_id' => $field['#form_builder']['element_id'],
        'element_type' => 'question',
        'configurable' => TRUE,
        'removable' => TRUE,
        'parent_id' => 0,
      ),
      '#weight' => $field['#weight'],
    );

    if ($options !== FALSE) {
      $field['#options'] = $options;
    }

    if ($label !== FALSE) {
      $field['#title'] = $label;
    }

  }

}

/**
 * Complete survey form callback
 */
function survey_builder_complete_form($form, &$form_state, $form_id) {

  module_load_include('inc', 'survey_builder', 'includes/survey_builder.survey');

  $survey = survey_load($form_id);
  if (!$survey) {
    drupal_not_found();
    exit();
  }

  drupal_set_title($survey->title);

  $form_state['survey_id'] = $form_id;

  $form = $survey->fields;
  foreach ($form as $key => &$field) {
    survey_builder_load_process_field($field);
  }

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Submit'),
    '#weight' => 1000,
  );

  return $form;

}

/**
 * Complete survey form submit handler
 */
function survey_builder_complete_form_submit($form, &$form_state) {

  // Save survey response entity and associated question response entities
  module_load_include('inc', 'survey_builder', 'includes/survey_builder.survey');

  $response_id = survey_response_save($form_state['survey_id'], $form_state['input']);
  if (!$response_id) {
    drupal_set_message(t('Unable to save survey response'), 'error');
  }

  $form_state['redirect'] = 'surveys';

}

/**
 * View completed survey page callback
 */
function survey_builder_response_view($response_id) {

  module_load_include('inc', 'survey_builder', 'includes/survey_builder.survey');

  $response = survey_response_load($response_id);
  if (!$response) {
    return MENU_NOT_FOUND;
  }

  $survey = survey_load($response->survey_id);
  if (!$survey) {
    return MENU_NOT_FOUND;
  }

  $header = array(
    t('Question'),
    t('Response')
  );

  $rows = survey_builder_response_process_fields($survey->fields, $response->pid);

  return theme('table', array('header' => $header, 'rows' => $rows));

}

function survey_builder_response_process_fields($fields, $survey_response_id) {

  $rows = array();

  foreach ($fields as $key => &$field) {

    if (strpos($key, '#') === 0) {
      continue;
    }
    elseif (isset($field['#type']) && $field['#type'] == 'fieldset') {
      // Recursively process any fields within the fieldset
      $rows = array_merge($rows, survey_builder_response_process_fields($field, $survey_response_id));
    }
    else {

      survey_builder_load_process_field($field, $survey_response_id);

      if (!isset($field['#default_value'])) {
        $value = '';
      }
      elseif (is_array($field['#default_value'])) {
        $value = theme('item_list', array('items' => $field['#default_value']));
      }
      else {
        $value = check_plain($field['#default_value']);
      }

      $rows[] = array($field['#title'], $value);

    }

  }

  return $rows;

}
